//------------------------------------------------------------------------------
// LegoEx1.cs
//
//     This code was generated by the DssNewService tool.
//
//------------------------------------------------------------------------------
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml;
using legoex1 = Robotics.LegoEx1;
using bumper = Microsoft.Robotics.Services.ContactSensor.Proxy;
using drive = Microsoft.Robotics.Services.Drive.Proxy;
using gamepad = Microsoft.Robotics.Services.Sample.XInputGamepad.Proxy;

namespace Robotics.LegoEx1
{
    
    /// <summary>
    /// The Lego Ex1 Service
    /// </summary>
    [DisplayName("LegoEx1")]
    [Description("The LegoEx1 Service")]
    [Contract(Contract.Identifier)]
    public class LegoEx1Service : DsspServiceBase
    {
        /// <summary>
        /// _state
        /// </summary>
        private LegoEx1State _state = new LegoEx1State();
        /// <summary>
        /// _main Port
        /// </summary>
        [ServicePort("/legoex1", AllowMultipleInstances=false)]
        private LegoEx1Operations _mainPort = new LegoEx1Operations();

		// partnerships with bumper, differential drive and gamepad
		[Partner("bumper", Contract=bumper.Contract.Identifier, CreationPolicy=PartnerCreationPolicy.UseExisting)]
		private bumper.ContactSensorArrayOperations _bumperPort = new bumper.ContactSensorArrayOperations();

		[Partner("drive", Contract=drive.Contract.Identifier, CreationPolicy=PartnerCreationPolicy.UseExisting)]
		private drive.DriveOperations _drivePort = new drive.DriveOperations();

		[Partner("XInputGamepad", Contract=gamepad.Contract.Identifier, CreationPolicy=PartnerCreationPolicy.CreateAlways)]
		private gamepad.XInputGamepadOperations _gamepadPort = new gamepad.XInputGamepadOperations();


        /// <summary>
        /// Default Service Constructor
        /// </summary>
        public LegoEx1Service(DsspServiceCreationPort creationPort) : 
                base(creationPort)
        {
        }
        /// <summary>
        /// Service Start
        /// </summary>
        protected override void Start()
        {
			base.Start();
			// Add service specific initialization here.

			// subscribe to bumper notifications
			bumper.ContactSensorArrayOperations bumperNotify = new bumper.ContactSensorArrayOperations();
			_bumperPort.Subscribe(bumperNotify);
			Activate(Arbiter.Receive<bumper.Update>(true, bumperNotify, BumperHandler));

			// subscribe to button presses and thumbstick changes on the 360 pad
			gamepad.XInputGamepadOperations gamepadNotify = new gamepad.XInputGamepadOperations();
			_gamepadPort.Subscribe(gamepadNotify);
			Activate(Arbiter.Receive<gamepad.ThumbsticksChanged>(true, gamepadNotify, ThumbstickHandler));
			Activate(Arbiter.Receive<gamepad.ButtonsChanged>(true, gamepadNotify, ButtonHandler));
        }

		private void ThumbstickHandler(gamepad.ThumbsticksChanged msg)
		{
			if(_state.MotorEnabled)
			{
				// Left/RightWheelPower expects a value from -1.0f to 1.0f.
				// the Thumbsticks will return a value form -1.0f to 1.0f.  Convenient.

				// create a request
				drive.SetDrivePowerRequest req = new drive.SetDrivePowerRequest();

				// assign the values
				req.LeftWheelPower = msg.Body.LeftY;
				req.RightWheelPower = msg.Body.RightY;

				// post the request
				_drivePort.SetDrivePower(req);
			}
		}

		private void ButtonHandler(gamepad.ButtonsChanged msg)
		{
			if(msg.Body.A)
				_state.MotorEnabled = true;
		}

		private void BumperHandler(bumper.Update msg)
		{
			if(msg.Body.Pressed)
			{
				LogInfo("Pressed!");

				// create a request
				drive.SetDrivePowerRequest req = new drive.SetDrivePowerRequest();

				// stop the wheels
				req.LeftWheelPower = 0.0f;
				req.RightWheelPower = 0.0f;

				// post the request
				_drivePort.SetDrivePower(req);
			}
		}

        /// <summary>
        /// Get Handler
        /// </summary>
        /// <param name="get"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> GetHandler(Get get)
        {
            get.ResponsePort.Post(_state);
            yield break;
        }
    }
}
